@@ -1,5 +1,7 @@
-package Text::MultiMarkdown;
 require 5.008_000;
+use utf8;
+
+package Text::MultiMarkdown;
 use strict;
 use warnings;
 use re 'eval';
@@ -9,10 +11,13 @@ use Encode         qw();
 use Carp           qw(carp croak);
 use base           qw(Text::Markdown);
 use HTML::Entities qw(encode_entities);
+use Scalar::Util   qw(blessed);
+use Unicode::Normalize ();
 
-our $VERSION   = '1.002';
-$VERSION = eval $VERSION;
-our @EXPORT_OK = qw(markdown);
+our $VERSION   = '1.003';
+our @EXPORT_OK = qw(markdown multimarkdown_to_html);
+
+=encoding utf8
 
 =head1 NAME
 
@@ -20,21 +25,25 @@ Text::MultiMarkdown - Convert MultiMarkdown syntax to (X)HTML
 
 =head1 SYNOPSIS
 
-    use Text::MultiMarkdown 'markdown';
-    my $html = markdown($text);
+Use it as a function, with or without optional arguments:
 
     use Text::MultiMarkdown 'markdown';
+
+    my $html = markdown($text);
+
     my $html = markdown( $text, {
         empty_element_suffix => '>',
         tab_width => 2,
         use_wikilinks => 1,
     } );
 
+Or in the object-oriented interface:
+
     use Text::MultiMarkdown;
+
     my $m = Text::MultiMarkdown->new;
     my $html = $m->markdown($text);
 
-    use Text::MultiMarkdown;
     my $m = Text::MultiMarkdown->new(
         empty_element_suffix => '>',
         tab_width => 2,
@@ -54,7 +63,7 @@ specifically to serve as a front-end to (X)HTML. You can use span-level
 HTML tags anywhere in a Markdown document, and you can use block level
 HTML tags (C<< <div> >>, C<< <table> >> etc.). Note that by default
 Markdown isn't interpreted in HTML block-level elements, unless you add
-a C<markdown=1"> attribute to the element. See L<Text::Markdown> for
+a C<markdown="1"> attribute to the element. See L<Text::Markdown> for
 details.
 
 This module implements the MultiMarkdown markdown syntax extensions from:
@@ -80,9 +89,9 @@ and borrows from php-markdown, which lives at:
 This documentation is going to be moved/copied into this module for
 clearer reading in a future release..
 
-=head1 OPTIONS
+=head2 Options
 
-MultiMarkdown supports a number of options to it's processor which
+MultiMarkdown supports a number of options to its processor which
 control the behaviour of the output document.
 
 These options can be supplied to the constructor, on in a hash with
@@ -91,52 +100,12 @@ examples of both of the above styles.
 
 The options for the processor are:
 
-=over
-
-=item use_metadata
-
-Controls the metadata options below.
-
-=item strip_metadata
-
-If true, any metadata in the input document is removed from the output
-document (note - does not take effect in complete document format).
-
-=item empty element suffix
-
-This option can be used to generate normal HTML output. By default, it
-is ' />', which is xHTML, change to '>' for normal HTML.
-
-=item img_ids
-
-Controls if <img> tags generated have an id attribute. Defaults to true.
-Turn off for compatibility with the original markdown.
-
-=item heading_ids
-
-Controls if <hX> tags generated have an id attribute. Defaults to true.
-Turn off for compatibility with the original markdown.
-
-=item heading_ids_spaces_to_dash
-
-Controls whether spaces in headings should be rendered as "-" characters in the heading ids (for compatibility with github markdown, and others)
+=over 4
 
 =item bibliography_title
 
 The title of the generated bibliography, defaults to 'Bibliography'.
 
-=item tab_width
-
-Controls indent width in the generated markup, defaults to 4
-
-=item disable_tables
-
-If true, this disables the MultiMarkdown table handling.
-
-=item disable_footnotes
-
-If true, this disables the MultiMarkdown footnotes handling.
-
 =item disable_bibliography
 
 If true, this disables the MultiMarkdown bibliography/citation handling.
@@ -145,27 +114,63 @@ If true, this disables the MultiMarkdown bibliography/citation handling.
 
 If true, this disables the MultiMarkdown definition list handling.
 
-=back
+=item disable_footnotes
 
-A number of possible items of metadata can also be supplied as
-options. Note that if the use_metadata is true then the metadata in
-the document will overwrite the settings on command line.
+If true, this disables the MultiMarkdown footnotes handling.
 
-Metadata options supported are:
+=item disable_tables
 
-=over
+If true, this disables the MultiMarkdown table handling.
 
-=item document_format
+=item empty_element_suffix
 
-=item use_wikilinks
+This option can be used to generate normal HTML output. By default, it
+is C<< /> >>, which is xHTML, change to C<< > >> for normal HTML.
 
-=item base_url
+=item heading_ids
 
-=item self_url - The document url is prepended to the "#" anchor of footnotes.
+Controls if C<hX> tags generated have an id attribute. Defaults to true.
+Turn off for compatibility with the original markdown.
+
+=item heading_ids_spaces_to_dash
+
+Controls whether spaces in headings should be rendered as "-" characters
+in the heading ids (for compatibility with GitHub markdown, and others)
+
+=item img_ids
+
+Controls if C<img> tags generated have an id attribute. Defaults to true.
+Turn off for compatibility with the original markdown.
+
+=item strip_metadata
+
+If true, any metadata in the input document is removed from the output
+document (note - does not take effect in complete document format).
+
+=item tab_width
+
+Controls indent width in the generated markup, defaults to 4
+
+=item transliterated_ids
+
+In markdown label values, change accented and other non-ASCII letter
+characters with L<Text::Unidecode>. If that module is not available,
+this issues a warning and does nothing. When C<unicode_ids> is true,
+this is ignored. The default is false.
+
+=item unicode_ids
+
+In markdown label values, allow any Unicode letter character along
+with the allowed ASCII symbol characters. This overrules
+C<transliterated_ids> when true. The default is false.
+
+=item use_metadata
+
+Controls the metadata options below.
 
 =back
 
-=head1 METADATA
+=head2 Metadata
 
 MultiMarkdown supports the concept of 'metadata', which allows you to
 specify a number of formatting options within the document itself.
@@ -178,18 +183,24 @@ or options to the markdown method itself. (Note, as metadata, keys
 contain space, whereas options the keys are underscore separated.)
 
 You can attach arbitrary metadata to a document, which is output in
-HTML <META> tags if unknown, see t/11document_format.t for more info.
+HTML C<< <META> >> tags if unknown, see F<t/11-document_format.t> for
+an example.
 
-A list of 'known' metadata keys, and their effects are listed below:
+These are the known metadata keys:
 
-=over
+=over 4
 
 =item document_format
 
 If set to 'complete', MultiMarkdown will render an entire xHTML page,
 otherwise it will render a document fragment
 
-=over
+=over 4
+
+=item base url
+
+This is the base URL for referencing wiki pages. In this is not
+supplied, all wiki links are relative.
 
 =item css
 
@@ -206,16 +217,13 @@ Sets the page title, if in 'complete' document format.
 If set to '1' or 'on', causes links that are WikiWords to
 automatically be processed into links.
 
-=item base url
-
-This is the base URL for referencing wiki pages. In this is not
-supplied, all wiki links are relative.
-
 =back
 
-=head1 METHODS
+=head2 Class methods
 
-=head2 new
+=over 4
+
+=item new
 
 A simple constructor, see the SYNTAX and OPTIONS sections for more information.
 
@@ -247,47 +255,261 @@ sub new {
 	my @binary = qw(use_metadata use_wikilinks);
 	$p{$_} = $p{$_} ? 1 : 0 for @binary;
 
-	unless( $p{tab_width} =~ m/^\d+$/ ) {
+	unless( $p{tab_width} =~ m/^[0-9]+$/ ) {
 		carp "tab_width did not look like a decimal number, so using the default 4";
 		$p{tab_width} = 4;
 	}
 
+	_process_id_handler( \%args, \%p );
+
     my $self = { params => \%p };
     bless $self, ref($class) || $class;
     return $self;
 }
 
-=head2 markdown
+sub _id_handler {
+	defined $_[0]->{id_handler} ? $_[0]->{id_handler} : \&_default_id_handler
+}
 
-The main function as far as the outside world is concerned. See the SYNOPSIS
-for details on use.
+sub _default_id_handler {
+	my ($label) = @_;
+
+	$label =~ s/[^A-Za-z0-9:_.-]//g;
+	$label =~ s/\A[^A-Za-z]+//g;
+	$label =~ s/-+/-/g;
+	$label =~ s/-+\z//g;
+
+	return $label;
+}
+
+BEGIN {
+my $has_unidecode = eval { require Text::Unidecode };
+
+sub _transliteration_id_handler {
+	my ($label) = @_;
+
+	unless ($has_unidecode ) {
+		carp "Need Text::Unidecode to for transliterated_ids, but could not load it. Falling back to default id handler";
+		return _default_id_handler($label);
+	}
+
+	$label = Text::Unidecode::unidecode($label);
+
+	$label =~ s/\s+//g;
+	$label =~ s/\A[^A-Za-z]+//g;
+	$label =~ s/-+/-/g;
+	$label =~ s/-+\z//g;
+
+	return $label;
+}
+
+{
+no warnings qw(redefine);
+*_transliteration_id_handler = \&_default_id_handler unless $has_unidecode;
+}
+
+sub _unicode_id_handler {
+	my ($label) = @_;
+
+	$label =~ s/\s+//g;
+	$label =~ s/\W+/-/g;
+	$label =~ s/\A\P{Letter}+//g;
+	$label =~ s/-+/-/g;
+	$label =~ s/-+\z//g;
+	return $label;
+}
+
+sub _process_id_handler {
+	my( $args, $p ) = @_;
+
+	$p->{id_handler} = \&_default_id_handler;
+
+	if ( exists $args->{unicode_ids} and $args->{unicode_ids} and exists $args->{transliterated_ids} ) {
+		warn "ignoring transliterated_ids because unicode_ids is true\n";
+		delete $args->{transliterated_ids};
+		}
+
+	if ( $args->{unicode_ids} ) {
+		$p->{id_handler} = \&_unicode_id_handler
+		}
+	elsif ( $args->{transliterated_ids} ) {
+		warn "Need Text::Unidecode to transliterate labels, but could not load it\n"
+			unless $has_unidecode;
+		$p->{id_handler} = \&_transliteration_id_handler;
+		}
+	}
+}
+
+=back
+
+=head2 Instance methods
+
+=over 4
+
+=item markdown( MARKDOWN_TEXT [, HASHREF] )
+
+This is the legacy interface to this module, but it does too much and
+is a poor name. For the function form, use C<multimarkdown_to_html>
+instead. At the moment that's just a wrapper for C<markdown> in the
+functional form. For the object-oriented forms, use C<to_html> instead.
+That's also just a wrapper for this, but will later change to enforce
+object-orientedness (i.e. exclude the functional form).
+
+And now the legacy stuff.
+
+This works as either a class method, instance method, or exportable
+function:
+
+	my $html = Text::MultiMarkdown->markdown( $text );
+
+	my $mm = Text::MultiMarkdown->new;
+	my $html = $mm->markdown($text);
+
+	use Text::MultiMarkdown qw(markdown);
+	my $html = markdown( $text );
+
+Any of these forms take an optional HASH_REF argument for options. These
+are the options for this module or the parent class L<Text::Markdown>:
+
+	my $html = Text::MultiMarkdown->markdown( $text, { ... } );
+
+	my $mm = Text::MultiMarkdown->new;
+	my $html = $mm->markdown($text, { ... });
+
+	use Text::MultiMarkdown qw(markdown);
+	my $html = markdown( $text, { ... } );
+
+To make this work in all these cases, since this was the legacy design,
+various unsavory things have to happen.
+
+When called as a class method, a new object is constructed. We guess
+that it's a class method by looking at the first argument and seeing
+that it looks like a Perl package name. In prior versions this was
+documented to not work, but there was also a TODO test for it to work.
+So, now it works. This might fail if the entire markdown text is exactly
+a valid Perl package name.
+
+If the first argument is a blessed reference, we guess that this is
+an instance method. With the optional HASH_REF argument this constructs
+a new argument with all of the settings of the original object and the
+stuff in HASH_REF. This might fail if you have some weird case where
+you call this as a function but pass as the TEXT argument an object that
+has overloaded stringification .
 
 =cut
 
+=begin comment
+
+=end comment
+
+There are these situations:
+
+	CLASS->markdown( TEXT );
+	CLASS->markdown( TEXT, HASHREF );
+
+	OBJ->markdown( TEXT );
+	OBJ->markdown( TEXT, HASHREF );
+
+	markdown( TEXT );
+	markdown( TEXT, HASHREF );
+
+These are really:
+
+	markdown( CLASS, TEXT )
+	markdown( CLASS, TEXT, HASHREF )
+
+	markdown( OBJ, TEXT )
+	markdown( OBJ, TEXT, HASHREF )
+
+	markdown( TEXT );
+	markdown( TEXT, HASHREF );
+
+Which breaks down to these groups:
+
+	1) markdown( TEXT );
+
+	2.1) markdown( TEXT, HASHREF );
+	2.2) markdown( CLASS, TEXT )
+	2.3) markdown( OBJ, TEXT )
+
+	3.1) markdown( CLASS, TEXT, HASHREF )
+	3.2) markdown( OBJ, TEXT, HASHREF )
+
+In 1), 2.2), and 3.1), we should make a new object and then do our
+thing.
+
+In 3.1), the previous version specifically said that we can't call
+this as a class method.
+
+In 3.2), we need to merge the options in the existing object with
+the new options. This was never a documented feature though.
+
+Part of the tickyness is that interface for Text::Markdown. We need
+to pass the HASHREF to _CleanUpRunData in the SUPER class
+
+=cut
+
+sub _looks_like_class {
+	local $_ = $_[0];
+	m/\A\w+(?:::\w+)+\z/;
+}
+
 sub markdown {
-    my ( $self, $text, $options ) = @_;
+	my( $self, $text, $options ) = do {
+		if ( @_ == 1 and ! ref $_[0] ) { # Case 1
+			( __PACKAGE__->new, $_[0], {} );
+		} elsif ( @_ == 2 and ! _looks_like_class($_[0]) and ref $_[1] eq ref {} ) { # Case 2.1
+			( __PACKAGE__->new( %{ $_[1] } ), $_[0], $_[1] );
+		} elsif ( @_ == 2 and _looks_like_class($_[0]) and ! ref $_[1] ) { # Case 2.2
+			( $_[0]->new, $_[1] );
+		} elsif ( @_ == 2 and blessed($_[0]) and ! ref $_[1] ) { # Case 2.3
+			( $_[0], $_[1], {} );
+		} elsif ( @_ == 3 and _looks_like_class($_[0]) and ! ref $_[1] and ref $_[2] eq ref {} ) { # Case 3.1
+			( $_[0]->new( %{ $_[2]} ), $_[1], $_[2] );
+		} elsif ( @_ == 3 and blessed($_[0]) and ! ref $_[1] and ref $_[2] eq ref {} ) { # Case 3.2
+			my %merged = ( %{ $_[0]->{params} }, %{ $_[2] } );
+			my $new = $_[0]->new( %merged );
+			( $new, $_[1], $_[2] );
+		} else {
+			carp "Unrecognized arguments for markdown()";
+			return;
+		}
+	};
 
-    # Detect functional mode, and create an instance for this run..
-    unless (ref $self) {
-        if ( $self ne __PACKAGE__ ) {
-            my $ob = __PACKAGE__->new();
-                                # $self is text, $text is options
-            return $ob->markdown($self, $text);
-        }
-        else {
-            croak('Calling ' . $self . '->markdown (as a class method) is not supported.');
-        }
-    }
+    $options = {} unless defined $options;
 
-    $options ||= {};
-
-    %$self = (%{ $self->{params} }, %$options, params => $self->{params});
-
-    $self->_CleanUpRunData($options);
+	%$self = (%{ $self->{params} }, %$options, params => $self->{params});
+	$self->_CleanUpRunData($options);
 
     return $self->_Markdown($text);
 }
 
+=item multimarkdown_to_html
+
+For the functional interface, you should use this instead of C<markdown>
+because it's a better name. At the moment it's the same as calling
+C<markdown>, but eventually this will diverge from the object-oriented
+form C<to_html>, which is also a better name.
+
+=cut
+
+sub multimarkdown_to_html {
+	markdown(@_);
+}
+
+=item to_html
+
+As a class or instance method, you should use this instead of C<markdown>
+because it's a better name. At the moment it's the same as calling
+C<markdown>, but eventually this will diverge from the functional
+form C<multimarkdown_to_html>, which is also a better name.
+
+=cut
+
+sub to_html {
+	markdown(@_);
+}
+
 sub _CleanUpRunData {
     my ($self, $options) = @_;
     # Clear the global hashes. If we don't clear these, you get conflicts
@@ -855,10 +1077,8 @@ sub _Header2Label {
     my ($self, $header) = @_;
     my $label = lc $header;
     $label =~ s/ +/-/g if $self->{heading_ids_spaces_to_dash};
-    $label =~ s/[^A-Za-z0-9:_.-]//g;        # Strip illegal characters
-    while ($label =~ s/^[^A-Za-z]//g)
-        {};     # Strip illegal leading characters
-    return $label;
+
+	return $self->_id_handler->($label);
 }
 
 sub _Id2Footnote {
@@ -1330,7 +1550,7 @@ sub _PrintMarkdownBibliography {
 
 1;
 
-__END__
+=back
 
 =head1 BUGS
 
@@ -1366,7 +1586,7 @@ See the Changes file for detailed release notes for this version.
 =head1 THIS DISTRIBUTION
 
 Please note that this distribution is a fork of Fletcher Penny's MultiMarkdown project,
-and it *is not* in any way blessed by him.
+and it I<is not> in any way blessed by him.
 
 Whilst this code aims to be compatible with the original MultiMarkdown (and incorporates
 and passes the MultiMarkdown test suite) whilst fixing a number of bugs in the original -
