@@ -1,10 +1,6 @@
-package TestUtils;
 use warnings;
 use strict;
 
-use Exporter qw(import);
-our @EXPORT = qw(difftest get_files run_tests slurp tidy);
-
 use Encode;
 use File::Spec::Functions qw(catfile);
 use List::MoreUtils qw(uniq);
@@ -23,8 +19,8 @@ BEGIN {
 				pass($testname);
 				return 1;
 			}
-			warn "=" x 80 . "\nTest <$testname> DIFFERENCES: + = processed version from .text, - = template from .html\n";
-			warn encode('utf8', Text::Diff::diff(\$expected => \$got, { STYLE => "Unified" }) . "\n");
+			print "=" x 80 . "\nTest <$testname> DIFFERENCES: + = processed version from .text, - = template from .html\n";
+			print encode('utf8', Text::Diff::diff(\$expected => \$got, { STYLE => "Unified" }) . "\n");
 			fail($testname);
 			return 0;
 		};
